
<%def name="make_wutta_components()">
  ${self.make_wutta_request_mixin()}
  ${self.make_wutta_button_component()}
  ${self.make_wutta_datepicker_component()}
  ${self.make_wutta_timepicker_component()}
  ${self.make_wutta_filter_component()}
  ${self.make_wutta_filter_value_component()}
  ${self.make_wutta_filter_date_value_component()}
  ${self.make_wutta_tool_panel_component()}
</%def>

<%def name="make_wutta_request_mixin()">
  <script>

    const WuttaRequestMixin = {
        methods: {

            wuttaGET(url, params, success, failure) {

                this.$http.get(url, {params: params}).then(response => {

                    if (response.data.error) {
                        this.$buefy.toast.open({
                            message: `Request failed:  ${'$'}{response.data.error}`,
                            type: 'is-danger',
                            duration: 4000, // 4 seconds
                        })
                        if (failure) {
                            failure(response)
                        }

                    } else {
                        success(response)
                    }

                }, response => {
                    this.$buefy.toast.open({
                        message: "Request failed:  (unknown server error)",
                        type: 'is-danger',
                        duration: 4000, // 4 seconds
                    })
                    if (failure) {
                        failure(response)
                    }
                })

            },

            wuttaPOST(action, params, success, failure) {

                const csrftoken = ${json.dumps(h.get_csrf_token(request))|n}
                const headers = {'X-CSRF-TOKEN': csrftoken}

                this.$http.post(action, params, {headers: headers}).then(response => {

                    if (response.data.error) {
                        this.$buefy.toast.open({
                            message: "Submit failed:  " + (response.data.error ||
                                                           "(unknown error)"),
                            type: 'is-danger',
                            duration: 4000, // 4 seconds
                        })
                        if (failure) {
                            failure(response)
                        }

                    } else {
                        success(response)
                    }

                }, response => {
                    this.$buefy.toast.open({
                        message: "Submit failed!  (unknown server error)",
                        type: 'is-danger',
                        duration: 4000, // 4 seconds
                    })
                    if (failure) {
                        failure(response)
                    }
                })
            },
        },
    }

  </script>
</%def>

<%def name="make_wutta_button_component()">
  <script type="text/x-template" id="wutta-button-template">
    <b-button :type="type"
              :native-type="nativeType"
              :tag="tag"
              :href="href"
              :title="title"
              :disabled="buttonDisabled"
              @click="clicked"
              icon-pack="fas"
              :icon-left="iconLeft">
      {{ buttonLabel }}
    </b-button>
  </script>
  <script>
    const WuttaButton = {
        template: '#wutta-button-template',
        props: {
            type: String,
            nativeType: String,
            tag: String,
            href: String,
            label: String,
            title: String,
            iconLeft: String,
            working: String,
            workingLabel: String,
            disabled: Boolean,
            once: Boolean,
        },
        data() {
            return {
                currentLabel: null,
                currentDisabled: null,
            }
        },
        computed: {
            buttonLabel: function() {
                return this.currentLabel || this.label
            },
            buttonDisabled: function() {
                if (this.currentDisabled !== null) {
                    return this.currentDisabled
                }
                return this.disabled
            },
        },
        methods: {

            clicked(event) {
                if (this.once) {
                    this.currentDisabled = true
                    if (this.workingLabel) {
                        this.currentLabel = this.workingLabel
                    } else if (this.working) {
                        this.currentLabel = this.working + ", please wait..."
                    } else {
                        this.currentLabel = "Working, please wait..."
                    }
                }
            }
        },
    }
    Vue.component('wutta-button', WuttaButton)
  </script>
</%def>

<%def name="make_wutta_datepicker_component()">
  <script type="text/x-template" id="wutta-datepicker-template">
    <b-datepicker :name="name"
                  ref="datepicker"
                  :editable="editable"
                  icon-pack="fas"
                  icon="calendar-alt"
                  :date-formatter="formatDate"
                  :value="buefyValue"
                  :size="size"
                  @input="val => $emit('input', val)" />
  </script>
  <script>
    const WuttaDatepicker = {
        template: '#wutta-datepicker-template',
        props: {
            name: String,
            value: String,
            size: String,
            editable: {
                type: Boolean,
                default: true,
            },
        },
        data() {
            return {
                buefyValue: this.parseDate(this.value),
            }
        },
        methods: {

            focus() {
                this.$refs.datepicker.focus()
            },

            formatDate(date) {
                if (date === null) {
                    return null
                }
                // just need to convert to simple ISO date format here, seems
                // like there should be a more obvious way to do that?
                var year = date.getFullYear()
                var month = date.getMonth() + 1
                var day = date.getDate()
                month = month < 10 ? '0' + month : month
                day = day < 10 ? '0' + day : day
                return year + '-' + month + '-' + day
            },

            parseDate(date) {
                if (!date) {
                    return
                }
                if (typeof(date) == 'string') {
                    // nb. this assumes classic YYYY-MM-DD (ISO) format
                    var parts = date.split('-')
                    return new Date(parts[0], parseInt(parts[1]) - 1, parts[2])
                }
                return date
            },
        },
    }
    Vue.component('wutta-datepicker', WuttaDatepicker)
  </script>
</%def>

<%def name="make_wutta_timepicker_component()">
  <script type="text/x-template" id="wutta-timepicker-template">
    <b-timepicker :name="name"
                  editable
                  :value="buefyValue" />
  </script>
  <script>
    const WuttaTimepicker = {
        template: '#wutta-timepicker-template',
        props: {
            name: String,
            value: String,
            editable: {
                type: Boolean,
                default: true,
            },
        },
        data() {
            return {
                buefyValue: this.parseTime(this.value),
            }
        },
        methods: {

            formatTime(time) {
                if (time === null) {
                    return null
                }

                let h = time.getHours()
                let m = time.getMinutes()
                let s = time.getSeconds()

                h = h < 10 ? '0' + h : h
                m = m < 10 ? '0' + m : m
                s = s < 10 ? '0' + s : s

                return h + ':' + m + ':' + s
            },

            parseTime(time) {
                if (time.getHours) {
                    return time
                }

                let found, hours, minutes

                found = time.match(/^(\d\d):(\d\d):\d\d$/)
                if (found) {
                    hours = parseInt(found[1])
                    minutes = parseInt(found[2])
                    return new Date(null, null, null, hours, minutes)
                }

                found = time.match(/^\s*(\d\d?):(\d\d)\s*([AaPp][Mm])\s*$/)
                if (found) {
                    hours = parseInt(found[1])
                    minutes = parseInt(found[2])
                    const ampm = found[3].toUpperCase()
                    if (ampm == 'AM') {
                        if (hours == 12) {
                            hours = 0
                        }
                    } else { // PM
                        if (hours < 12) {
                            hours += 12
                        }
                    }
                    return new Date(null, null, null, hours, minutes)
                }
            },
        },
    }
    Vue.component('wutta-timepicker', WuttaTimepicker)
  </script>
</%def>

<%def name="make_wutta_filter_component()">
  <script type="text/x-template" id="wutta-filter-template">
    <div v-show="filter.visible"
         class="wutta-filter">

      <b-button @click="filter.active = !filter.active"
                class="filter-toggle"
                icon-pack="fas"
                :icon-left="filter.active ? 'check' : null"
                :size="isSmall ? 'is-small' : null">
        {{ filter.label }}
      </b-button>

      <div v-show="filter.active"
           style="display: flex; gap: 0.5rem;">

        <b-button v-if="verbKnown"
                  class="filter-verb"
                  @click="verbChoiceInit()"
                  :size="isSmall ? 'is-small' : null">
          {{ verbLabel }}
        </b-button>

        <b-autocomplete v-if="!verbKnown"
                        ref="verbAutocomplete"
                        :data="verbOptions"
                        v-model="verbTerm"
                        field="verb"
                        :custom-formatter="formatVerb"
                        open-on-focus
                        keep-first
                        clearable
                        clear-on-select
                        @select="verbChoiceSelect"
                        icon-pack="fas"
                        :size="isSmall ? 'is-small' : null" />

        ## nb. only *ONE* of the following is used, per filter data type

        <wutta-filter-date-value v-if="filter.data_type == 'date'"
                                 v-model="filter.value"
                                 ref="filterValue"
                                 v-show="valuedVerb()"
                                 :is-small="isSmall" />

        <wutta-filter-value v-else
                            v-model="filter.value"
                            ref="filterValue"
                            v-show="valuedVerb()"
                            :is-small="isSmall" />

      </div>
    </div>
  </script>
  <script>

    const WuttaFilter = {
        template: '#wutta-filter-template',
        props: {
            filter: Object,
            isSmall: Boolean,
        },

        data() {
            return {
                verbKnown: !!this.filter.verb,
                verbLabel: this.filter.verb_labels[this.filter.verb],
                verbTerm: '',
            }
        },

        computed: {

            verbOptions() {

                // construct list of options
                const options = []
                for (let verb of this.filter.verbs) {
                    options.push({
                        verb,
                        label: this.filter.verb_labels[verb],
                    })
                }

                // parse list of search terms
                const terms = []
                for (let term of this.verbTerm.toLowerCase().split(' ')) {
                    term = term.trim()
                    if (term) {
                        terms.push(term)
                    }
                }

                // show all if no search terms
                if (!terms.length) {
                    return options
                }

                // only show filters matching all search terms
                return options.filter(option => {
                    let label = option.label.toLowerCase()
                    for (let term of terms) {
                        if (label.indexOf(term) < 0) {
                            return false
                        }
                    }
                    return true
                })

                return options
            },
        },

        methods: {

            focusValue() {
                this.$refs.filterValue.focus()
            },

            formatVerb(option) {
                return option.label || option.verb
            },

            verbChoiceInit(option) {
                this.verbKnown = false
                this.$nextTick(() => {
                    this.$refs.verbAutocomplete.focus()
                })
            },

            verbChoiceSelect(option) {
                this.filter.verb = option.verb
                this.verbLabel = option.label
                this.verbKnown = true
                this.verbTerm = ''
                this.focusValue()
            },

            valuedVerb() {
                /* return true if the current verb should expose value input(s) */

                // if filter has no "valueless" verbs, then all verbs should expose value inputs
                if (!this.filter.valueless_verbs) {
                    return true
                }

                // if filter *does* have valueless verbs, check if "current" verb is valueless
                if (this.filter.valueless_verbs.includes(this.filter.verb)) {
                    return false
                }

                // current verb is *not* valueless
                return true
            },
        }
    }

    Vue.component('wutta-filter', WuttaFilter)

  </script>
</%def>

<%def name="make_wutta_filter_value_component()">
  <script type="text/x-template" id="wutta-filter-value-template">
    <div class="wutta-filter-value">

      <b-input v-model="inputValue"
               ref="valueInput"
               @input="val => $emit('input', val)"
               :size="isSmall ? 'is-small' : null" />

    </div>
  </script>
  <script>

    const WuttaFilterValue = {
        template: '#wutta-filter-value-template',
        props: {
            value: String,
            isSmall: Boolean,
        },

        data() {
            return {
                inputValue: this.value,
            }
        },

        methods: {

            focus: function() {
                this.$refs.valueInput.focus()
            }
        },
    }

    Vue.component('wutta-filter-value', WuttaFilterValue)

  </script>
</%def>

<%def name="make_wutta_filter_date_value_component()">
  <script type="text/x-template" id="wutta-filter-date-value-template">
    <div class="wutta-filter-value">

      <wutta-datepicker v-model="inputValue"
                        ref="valueInput"
                        :size="isSmall ? 'is-small' : null"
                        @input="valueChanged" >
      </wutta-datepicker>

    </div>
  </script>
  <script>

    const WuttaFilterDateValue = {
        template: '#wutta-filter-date-value-template',
        props: {
            value: String,
            isSmall: Boolean,
        },

        data() {
            return {
                inputValue: this.value,
            }
        },

        methods: {

            focus() {
                this.$refs.valueInput.focus()
            },

            valueChanged(value) {
                if (value) {
                    value = this.$refs.valueInput.formatDate(value)
                }
                this.$emit('input', value)
            },
        },
    }

    Vue.component('wutta-filter-date-value', WuttaFilterDateValue)

  </script>
</%def>

<%def name="make_wutta_tool_panel_component()">
  <script type="text/x-template" id="wutta-tool-panel-template">
    <nav class="panel tool-panel">
      <p class="panel-heading">{{ heading }}</p>
      <div class="panel-block">
        <div style="display: flex; flex-direction: column; gap: 0.5rem;">
          <slot />
        </div>
      </div>
    </nav>
  </script>
  <script>

    const WuttaToolPanel = {
        template: '#wutta-tool-panel-template',
        props: {
            heading: String,
        },
    }

    Vue.component('wutta-tool-panel', WuttaToolPanel)

  </script>
</%def>
